
#ifndef __RENDERERINTERFACE_H_
#define __RENDERERINTERFACE_H_
//
// This file is part of the GameFramework project released by Wazoo Enterprises Inc.
// under the GNU Public License. 
//
// Any projects using this code must attach these copyright notices along with their
// own as outlined at http://www.gnu.org
//
// Copyright 2003 by Wazoo Enterprises Inc.
// To contact the auther, send a message to wazoo@wazooenterprises.com
//


#include <windows.h>
#include <string>

/**
* version 1.1
* Added our Vertex Buffer interface
**/
#include "vbInterface.h"

/**
* version 1.3
* Added our texture manager interface
**/
#include "textureManagerInterface.h"

/**
* This class is our "base class" for obtaining
* some sort of GraphicsAPI independance. 
* We'll use this interface object to declare
* "high-level" methods which our "low-level" DLL
* projects will implement.
*/
class rendererInterface
{
protected:
	HWND m_hWnd;         /** handle to our main window */
	DWORD m_dwWidth;     /** our resolution width      */
	DWORD m_dwHeight;    /** our resolution height     */
	DWORD m_dwDepth;     /** our resolution depth      */
	BOOL m_bWindowed;    /** flag for being windowed or fullscreen */

public:
	/**
	* v1.1
	* Added a pointer to our Vertex Buffer interface object
	*/
	vbInterface* m_pVB; /** pointer to our Vertex Buffer interface object */

	/**
	* v1.3
	* Added a pointer to our texture manager interface object
	*/
	textureManagerInterface* m_pTextureManager; /** pointer to our texture manager */


public:
	/**
	* Constructor
	*/
	rendererInterface(){};
	
	/**
	* Destructor
	*/
	virtual ~rendererInterface(){};

	/**
	* Method to create and initialize our hardware
	* @param HWND - our window handle
	* @param DWORD - our window width
	* @param DWORD - our window height
	* @param DWORD - our window depth
	* @param BOOL  - flag for being windowed or fullscreen
	* @return HRESULT - SUCCEEDED if we're ok, FAILED otherwise
	*/
    virtual HRESULT createDevice(HWND, DWORD, DWORD, DWORD, BOOL) = 0;

    /**
	* Method to destroy it
	* @param void
	* @return void
	*/
    virtual void destroyDevice(void) = 0;

    /**
	* Method to begin rendering primitives
	* @param void
	* @return HRESULT - SUCCEEDED if our hardware pipeline is ready, FAILED if not
	*/
    virtual HRESULT beginRenderingScene(void) = 0;

    /**
	* Method to end rendering primitives
	* @param void
	* @return void
	*/
    virtual void endRenderingScene(void) = 0;

    /**
	* Method to change the screen clearing color
	* @param FLOAT - red component
	* @param FLOAT - green component
	* @param FLOAT - blue component
	* @param FLOAT - alpha component
	* @return void
	*/
    virtual void setClearColor(FLOAT, FLOAT, FLOAT, FLOAT) = 0;

	/**
	* Method to return our pointer to our textureManagerInterface
	* @param void
	* @return textureManagerInterface* - pointer to our textureManagerInterface
	*/
	textureManagerInterface* getTextureInterface(void){ return m_pTextureManager;};



};

extern "C"
{
	HRESULT createRendererInterface(rendererInterface **pInterface);
	typedef HRESULT (*CREATERENDERERINTERFACE)(rendererInterface **pInterface);

	HRESULT destroyRendererInterface(rendererInterface **pInterface);
	typedef HRESULT (*DESTROYRENDERERINTERFACE)(rendererInterface **pInterface);
}

#endif